﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SetCBLUserImage
{
    public class ApplicationRecord : UserImageRecord
    {
        public byte MachineID { get; private set; }
        public byte DataTypeID { get; private set; }
        public string Name { get; private set; }
        public byte[] ApplicationData { get; private set; }

        public ApplicationRecord(byte machineID, byte dataTypeID, string name, byte[] data)
            : base(0x37AE, RecordType.Application, 0xB49D)
        {
            _Init(machineID, dataTypeID, name, data);
        }

        public ApplicationRecord(ushort marker, RecordType type, ushort unknown1, byte[] data)
            : base(marker, type, unknown1)
        {
            var name = ASCIIEncoding.ASCII.GetString(data, 2, 8).Trim();

            int size = data[10] | (data[11] << 8) | (data[12] << 16) | (data[13] << 24);
            var appData = new byte[size];
            Array.Copy(data, 14, appData, 0, appData.Length);

            _Init(data[0], data[1], name, appData);
        }

        public override byte[] GetSaveData()
        {
            var stream = new MemoryStream();

            //Write machine ID and command ID
            stream.Write(new byte[] { MachineID, DataTypeID }, 0, 2);

            //Write application name (space-padded to 8 bytes)
            var name = ASCIIEncoding.ASCII.GetBytes(Name.Substring(0, Math.Min(Name.Length, 8)).PadRight(8));
            stream.Write(name, 0, name.Length);

            //Calculate size as data + 4 (and make sure it's even)
            var dataSize = ApplicationData.Length + 4;
            if (dataSize % 2 == 1)
            {
                dataSize++;
            }

            //Build length bytes
            var final = new byte[dataSize];
            var size = new byte[4];
            size[0] = (byte)(ApplicationData.Length & 0xFF);
            size[1] = (byte)((ApplicationData.Length >> 8) & 0xFF);
            size[2] = (byte)((ApplicationData.Length >> 16) & 0xFF);
            size[3] = (byte)((ApplicationData.Length >> 24) & 0xFF);
            Array.Copy(size, 0, final, 0, size.Length);

            //Write program data
            Array.Copy(ApplicationData, 0, final, 4, ApplicationData.Length);
            stream.Write(final, 0, final.Length);

            return _WrapSaveData(stream.ToArray());
        }

        public override string ToString()
        {
            return Name;
        }

        private void _Init(byte machineID, byte dataTypeID, string name, byte[] data)
        {
            MachineID = machineID;
            DataTypeID = dataTypeID;

            Name = name;
            ApplicationData = data;
        }
    }
}
